@@ -298,8 +298,10 @@ namespace xt
         static constexpr bool contiguous_layout() { return E1::contiguous_layout && E2::contiguous_layout; }
         static constexpr bool convertible_types() { return std::is_convertible<e2_value_type, e1_value_type>::value; }
 
+        static constexpr bool use_xsimd() { return xt_simd::simd_traits<int8_t>::size > 1; }
+
         template <class T>
-        static constexpr bool simd_size_impl() { return xt_simd::simd_traits<T>::size > 1 || is_bool<T>::value; }
+        static constexpr bool simd_size_impl() { return xt_simd::simd_traits<T>::size > 1 || (is_bool<T>::value && use_xsimd()); }
         static constexpr bool simd_size() { return simd_size_impl<e1_value_type>() && simd_size_impl<e2_value_type>(); }
         static constexpr bool simd_interface() { return has_simd_interface<E2, requested_value_type>(); }
         static constexpr bool simd_assign() { return convertible_types() && simd_size() && simd_interface(); }
