@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -370,6 +371,8 @@ namespace xt
         using const_reverse_iterator = typename buffer_base_type::const_reverse_iterator;
         using temporary_type = uvector<value_type, allocator_type>;
 
+        xbuffer_adaptor() = default;
+
         using base_type::base_type;
 
         ~xbuffer_adaptor() = default;
@@ -447,6 +450,9 @@ namespace xt
         xiterator_adaptor(self_type&&) = default;
         xiterator_adaptor& operator=(self_type&&) = default;
 
+        xiterator_adaptor& operator=(const temporary_type& rhs);
+        xiterator_adaptor& operator=(temporary_type&& rhs);
+
         size_type size() const noexcept;
         void resize(size_type size);
         
@@ -913,6 +919,20 @@ namespace xt
     {
     }
 
+    template <class I, class CI>
+    inline auto xiterator_adaptor<I, CI>::operator=(const temporary_type& rhs) -> self_type&
+    {
+        resize(rhs.size());
+        std::copy(rhs.cbegin(), rhs.cend(), m_it);
+        return *this;
+    }
+
+    template <class I, class CI>
+    inline auto xiterator_adaptor<I, CI>::operator=(temporary_type&& rhs) -> self_type&
+    {
+        return (*this = rhs);
+    }
+    
     template <class I, class CI>
     inline auto xiterator_adaptor<I, CI>::size() const noexcept -> size_type
     {
